## TODOs

## Next

- Add contains() configuration to avoid reading from non-turtle documents, such as *.jpg or *.png
- Add contains() configuration to chunk requests? (it seems like that's already happening in SolidEngine though... maybe because of performanceissues?)
- toTurtle baseIRI param
    it('serializes to Turtle', async () => {
        // Arrange
        const documentUrl = fakeDocumentUrl();
        const person = new Person({ url: `${documentUrl}#it`, name: faker.random.word() }, true);

        // Act
        const turtle = await person.toTurtle();

        // Assert
        const statements = turtle.split('\n').map(line => line.trim()).filter(line => line.length > 0);

        // TODO use .toEqualTurtle?
        expect(statements).toHaveLength(4);
        expect(statements[0]).toEqual(`<${person.url}> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .`);
        expect(statements[1]).toEqual(`<${person.url}> <http://xmlns.com/foaf/0.1/name> "${person.name}" .`);
        expect(statements[2]).toEqual(`<${person.url}-metadata> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <https://vocab.noeldemartin.com/crdt/Metadata> .`);
        expect(statements[3]).toEqual(`<${person.url}-metadata> <https://vocab.noeldemartin.com/crdt/resource> <${person.url}> .`);
    });

    it.skip('serializes to Turtle using baseIRI', async () => {
        // Arrange
        const documentUrl = fakeDocumentUrl();
        const person = new Person({ url: `${documentUrl}#it`, name: faker.random.word() }, true);

        // Act
        const turtle = await person.toTurtle({ baseIRI: documentUrl });

        // Assert
        const statements = turtle.split('\n').map(line => line.trim()).filter(line => line.length > 0);

        expect(statements).toHaveLength(4);
        expect(statements[0]).toEqual('<#it> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .');
        expect(statements[1]).toEqual(`<#it> <http://xmlns.com/foaf/0.1/name> "${person.name}" .`);
        expect(statements[2]).toEqual('<#it-metadata> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <https://vocab.noeldemartin.com/crdt/Metadata> .');
        expect(statements[3]).toEqual('<#it-metadata> <https://vocab.noeldemartin.com/crdt/resource> <#it> .');
    });

## Later

- Replace document updates with text/n3 (it's the only one mentioned in the spec)
- Refactor DocumentContainsRelation classes to be self-descriptive instead (if it quacks like a duck, it is a duck)
- Default to false timestamps??
